<?php

namespace Drupal\monarch_bootstrap_layout;

use Drupal\Core\Layout\LayoutDefinition;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * The service for common monarch bootstrap layout functions.
 */
class MonarchBootstrapLayout {

  /**
   * The entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current_user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The current_route_match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $currentRouteMatch;

  /**
   * Constructs the service.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity_type.manager service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current_user service.
   * @param \Drupal\Core\Routing\RouteMatchInterface $current_route_match
   *   The current_route_match service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountProxyInterface $current_user, RouteMatchInterface $current_route_match) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->currentRouteMatch = $current_route_match;
  }

  /**
   * Get the current route user id.
   *
   * @return string|false
   *   Return the current route user id, or FALSE if there is no route user.
   */
  public function getRouteUserId() {
    $route_uid = FALSE;
    $route_parameters = $this->currentRouteMatch->getParameters();

    if (!$route_uid && $route_parameters->has('user')) {
      $route_uid = $route_parameters->get('user') ?: NULL;
    }

    if (!$route_uid && $route_parameters->has('uid')) {
      $route_uid = $route_parameters->get('uid') ?: NULL;
    }

    return $route_uid->uid->value ?? $route_uid;
  }

  /**
   * Check if a user matches the route user.
   *
   * @param string $uid
   *   The uid of the user to validate against the current route user.
   *
   * @return bool
   *   Returns TRUE if the uid matches the route user, or FALSE if not.
   */
  public function matchRouteUser(string $uid = 'current') : bool {
    $uid = ($uid === 'current' || $uid === 'self') ? ($this->currentUser->isAnonymous() ? NULL : $this->currentUser->id()) : $uid;
    $route_uid = $this->getRouteUserId();
    return $route_uid === FALSE ? TRUE : $uid == $route_uid;
  }

  /**
   * Is the current template in build mode?
   *
   * @param array $var
   *   The template context array.
   *
   * @return bool
   *   Returns TRUE if the template is in build mode, or FALSE if not.
   */
  public function inBuildMode(array $var) : bool {
    $regions = $var['layout']->getRegions() ?? [];

    foreach ($regions as $key => $region) {
      // If the layout builder is allowing us to add blocks, then show
      // everything.
      if (isset($var['content'][$key]['layout_builder_add_block'])) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Get all valid regions.
   *
   * @param \Drupal\Core\Layout\LayoutDefinition $layout
   *   The layout definition.
   * @param bool $build_mode
   *   Is the template in build mode?
   *
   * @return array
   *   The filtered array of valid regions from the layout definition.
   */
  public function getValidRegions(LayoutDefinition $layout = NULL, bool $build_mode = FALSE) : array {
    if (!$layout) {
      return [];
    }

    $regions = $layout->getRegions() ?? [];

    $ret = [];

    foreach ($regions as $key => $region) {
      // If the layout builder is allowing us to add blocks, then show
      // everything.
      if ($build_mode) {
        $ret[$key] = $region;
        continue;
      }

      $criteria_list = $region['criteria'] ?? [];
      // Add a default condition so tabs without any conditions appear by
      // default.
      $criteria_list[0][] = TRUE;

      foreach ($criteria_list as $criteria_group) {
        if (empty($criteria_group)) {
          continue;
        }

        foreach ($criteria_group as $criteria) {
          if (is_string($criteria)) {
            $criteria = array_map('trim', explode(':', $criteria));

            if (count($criteria) === 1) {
              $criteria = $this->currentUser->hasPermission($criteria[0]);
            }
            elseif (count($criteria) === 2) {
              if ($criteria[0] === 'permission') {
                $criteria = $this->currentUser->hasPermission($criteria[1]);
              }
              elseif ($criteria[0] === 'role') {
                $criteria = in_array($criteria[1], $this->currentUser->getRoles() ?: []);
              }
              elseif ($criteria[0] === 'user') {
                $criteria = $this->matchRouteUser($criteria[1]);
              }
              else {
                $criteria = FALSE;
              }
            }
            else {
              $criteria = FALSE;
            }
          }

          if (!$criteria) {
            continue 2;
          }
        }

        $ret[$key] = $region;
        break;
      }
    }

    return $ret;
  }

  /**
   * Preprocess tabs.
   *
   * @param array $var
   *   The template context array.
   */
  public function preprocessTabs(array &$var) : void {
    $validate_user = ($var['content']['#settings']['validate_user'] ?? NULL) ? $this->matchRouteUser() : TRUE;
    $build_mode = $this->inBuildMode($var);

    if ($validate_user || $build_mode) {
      $var['tabs'] = [];
      $regions = $this->getValidRegions($var['layout'], $build_mode);

      foreach ($regions as $key => $region) {
        $var['tabs'][$key] = $region['label'];
      }

      $var['layout_wrapper_class'] = array_filter(array_map('trim', explode(' ', $var['content']['#settings']['layout_wrapper_class'] ?? '')));
      $var['nav_wrapper_class'] = array_filter(array_map('trim', explode(' ', $var['content']['#settings']['nav_wrapper_class'] ?? '')));
      $var['tab_content_wrapper_class'] = array_filter(array_map('trim', explode(' ', $var['content']['#settings']['tab_content_wrapper_class'] ?? '')));

      switch ($var['content']['#settings']['tab_style'] ?? 'horizontal_tabs') {
        case 'vertical_pills':
          $var['tab_look'] = 'nav-pills';
          $var['tab_orientation'] = 'vertical';
          $var['tab_style'][] = 'vertical';
          $var['tab_style'][] = 'pills-style';
          $var['tab_style'][] = 'normal';
          break;

        case 'horizontal_tabs_justified':
          $var['tab_look'] = 'nav-tabs nav-justified';
          $var['tab_orientation'] = 'horizontal';
          $var['tab_style'][] = 'horizontal';
          $var['tab_style'][] = 'tabs-style';
          $var['tab_style'][] = 'justified';
          break;

        case 'horizontal_tabs_fill':
          $var['tab_look'] = 'nav-tabs nav-fill';
          $var['tab_orientation'] = 'horizontal';
          $var['tab_style'][] = 'horizontal';
          $var['tab_style'][] = 'tabs-style';
          $var['tab_style'][] = 'fill';
          break;

        case 'horizontal_pills':
          $var['tab_look'] = 'nav-pills';
          $var['tab_orientation'] = 'horizontal';
          $var['tab_style'][] = 'horizontal';
          $var['tab_style'][] = 'pills-style';
          $var['tab_style'][] = 'normal';
          break;

        case 'horizontal_pills_justified':
          $var['tab_look'] = 'nav-pills nav-justified';
          $var['tab_orientation'] = 'horizontal';
          $var['tab_style'][] = 'horizontal';
          $var['tab_style'][] = 'pills-style';
          $var['tab_style'][] = 'justified';
          break;

        case 'horizontal_pills_fill':
          $var['tab_look'] = 'nav-pills nav-fill';
          $var['tab_orientation'] = 'horizontal';
          $var['tab_style'][] = 'horizontal';
          $var['tab_style'][] = 'pills-style';
          $var['tab_style'][] = 'fill';
          break;

        default:
          $var['tab_look'] = 'nav-tabs';
          $var['tab_orientation'] = 'horizontal';
          $var['tab_style'][] = 'horizontal';
          $var['tab_style'][] = 'tabs-style';
          $var['tab_style'][] = 'normal';
          $var['tab_style'][] = preg_replace('/_/', '-', $var['content']['#settings']['tab_style'] ?? 'horizontal_tabs') . '-style';
          break;
      }
    }
    else {
      unset($var['content']);
    }
  }

}
